<?php

namespace Payroll\Attendance\Policies;

use App\User;
use Payroll\Attendance\Models\Attendance;

class AttendancePolicy
{

    /**
     * Determine if the given user can view the attendance.
     *
     * @param User $user
     * @param Attendance $attendance
     *
     * @return bool
     */
    public function view(User $user, Attendance $attendance)
    {
        if ($user->canDo('attendance.attendance.view') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('blocks.block.view') 
        && $user->is('manager')
        && $block->user->parent_id == $user->id) {
            return true;
        }

        return $user->id === $attendance->user_id;
    }

    /**
     * Determine if the given user can create a attendance.
     *
     * @param User $user
     * @param Attendance $attendance
     *
     * @return bool
     */
    public function create(User $user)
    {
        return  $user->canDo('attendance.attendance.create');
    }

    /**
     * Determine if the given user can update the given attendance.
     *
     * @param User $user
     * @param Attendance $attendance
     *
     * @return bool
     */
    public function update(User $user, Attendance $attendance)
    {
        if ($user->canDo('attendance.attendance.update') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('blocks.block.update') 
        && $user->is('manager')
        && $block->user->parent_id == $user->id) {
            return true;
        }

        return $user->id === $attendance->user_id;
    }

    /**
     * Determine if the given user can delete the given attendance.
     *
     * @param User $user
     * @param Attendance $attendance
     *
     * @return bool
     */
    public function destroy(User $user, Attendance $attendance)
    {
        if ($user->canDo('attendance.attendance.delete') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('blocks.block.delete') 
        && $user->is('manager')
        && $block->user->parent_id == $user->id) {
            return true;
        }

        return $user->id === $attendance->user_id;
    }

    /**
     * Determine if the given user can verify the given attendance.
     *
     * @param User $user
     * @param Attendance $attendance
     *
     * @return bool
     */
    public function verify(User $user, Attendance $attendance)
    {
        if ($user->canDo('attendance.attendance.verify') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('attendance.attendance.verify') 
        && $user->is('manager')
        && $attendance->user->parent_id == $user->id) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given attendance.
     *
     * @param User $user
     * @param Attendance $attendance
     *
     * @return bool
     */
    public function approve(User $user, Attendance $attendance)
    {
        if ($user->canDo('attendance.attendance.approve') && $user->is('admin')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperUser()) {
            return true;
        }
    }
}
